/**
 * Generate do for project
 * @function doc
 * @param {string} [dirname=process.cwd()] - Project directory name
 * @param {Object} [options={}] - Optional settings
 * @returns {Promise}
 */
'use strict'

const co = require('co')
const argx = require('argx')
const aglob = require('aglob')
const colorprint = require('colorprint')
const { parse } = require('react-docgen')
const { readFileAsync, writeFileAsync } = require('asfs')

/** @lends doc */
function doc (dirname = process.cwd(), options = {}) {
  let args = argx(arguments)
  options = args.pop('object') || {}
  dirname = args.shift('string') || process.cwd()

  let {
    pattern = 'lib/**/*.jsx',
    out = 'signature.json'
  } = options
  return co(function * () {
    let cwd = process.cwd()
    process.chdir(dirname)

    let data = {}
    colorprint.debug(`Parsing doc from ${pattern}...`)
    for (let filename of yield aglob(pattern)) {
      let code = String(yield readFileAsync(filename))
      try {
        let parsed = parse(code)
        let { displayName } = parsed
        data[ displayName ] = parsed
        colorprint.trace(`Component "${displayName}" found from ${filename}`)
      } catch (e) {
        colorprint.trace(`No component found from ${filename}`)
      }
    }
    yield writeFileAsync(out, JSON.stringify(data, null, 2))
    colorprint.debug(`Signature file generated: ${out}`)
    process.chdir(cwd)
  })
}

module.exports = doc
